@@ -1,6 +1,7 @@
 use naive_opt::Search;
 use regex::{Regex, RegexBuilder};
 use std::{
+    borrow::Cow,
     env::{self, Args},
     fs::File,
     io::{self, BufRead, BufReader, BufWriter, StdoutLock, Write},
@@ -23,7 +24,7 @@ enum MatchOn {
 }
 
 impl Config {
-    fn new(args: Args) -> Config {
+    fn new(args: Args) -> Self {
         let mut query = String::new();
         let mut filenames: Vec<String> = Vec::new();
         let mut max = 0;
@@ -36,10 +37,10 @@ impl Config {
                 if arg == "--" {
                     finished = true;
                     continue;
-                } else if arg.starts_with("-") {
-                    let trimmed = arg.trim_start_matches("-");
-                    if trimmed.starts_with("m=") {
-                        max = trimmed[2..].parse().unwrap_or(0);
+                } else if arg.starts_with('-') {
+                    let trimmed = arg.trim_start_matches('-');
+                    if let Some(stripped) = trimmed.strip_prefix("m=") {
+                        max = stripped.parse().unwrap_or(0);
                         continue;
                     }
                     for c in trimmed.chars() {
@@ -75,7 +76,7 @@ impl Config {
             error("No query specified");
         }
 
-        Config {
+        Self {
             query,
             filenames,
             max,
@@ -126,17 +127,19 @@ fn grep(cfg: Config) {
 
     let istty = atty::is(atty::Stream::Stdin);
 
-    let stdout = std::io::stdout().lock();
+    let stdout = std::io::stdout();
+    let stdout = stdout.lock();
     let mut writer = BufWriter::with_capacity(16384, stdout);
 
     if string_search {
         let query = if case_insensitive {
             query.to_lowercase()
         } else {
-            query.to_owned()
+            query
         };
         if !istty {
-            let mut stdin = io::stdin().lock();
+            let stdin = io::stdin();
+            let mut stdin = stdin.lock();
             let mut line = String::new();
             let mut i = 0;
             while let Ok(bytes) = stdin.read_line(&mut line) {
@@ -153,15 +156,14 @@ fn grep(cfg: Config) {
                     case_insensitive,
                     match_on,
                     i,
-                    &cleaned.to_owned(),
+                    cleaned,
                     "stdin",
                     &query,
-                ) {
-                    if max > 0 {
-                        matches += 1;
-                        if matches >= max {
-                            break;
-                        }
+                ) && max > 0
+                {
+                    matches += 1;
+                    if matches >= max {
+                        break;
                     }
                 }
 
@@ -171,15 +173,15 @@ fn grep(cfg: Config) {
             return;
         }
 
-        if filenames.len() == 0 {
+        if filenames.is_empty() {
             error("No files specified");
         }
 
+        let mut line = String::new();
         for filename in &filenames {
             let mut matches: u32 = 0;
-            let reader = &mut read_file(&filename);
+            let reader = &mut read_file(filename);
 
-            let mut line = String::new();
             let mut i = 0;
             while let Ok(bytes) = reader.read_line(&mut line) {
                 if bytes == 0 {
@@ -195,15 +197,14 @@ fn grep(cfg: Config) {
                     case_insensitive,
                     match_on,
                     i,
-                    &cleaned.to_owned(),
-                    &filename,
+                    cleaned,
+                    filename,
                     &query,
-                ) {
-                    if max > 0 {
-                        matches += 1;
-                        if matches >= max {
-                            break;
-                        }
+                ) && max > 0
+                {
+                    matches += 1;
+                    if matches >= max {
+                        break;
                     }
                 }
 
@@ -223,10 +224,10 @@ fn grep(cfg: Config) {
         let re = re.unwrap();
 
         if !istty {
-            let mut stdin = io::stdin().lock();
+            let stdin = io::stdin();
+            let mut stdin = stdin.lock();
             let mut line = String::new();
             let mut i = 0;
-            let re = re.clone();
             while let Ok(bytes) = stdin.read_line(&mut line) {
                 if bytes == 0 {
                     break;
@@ -239,15 +240,14 @@ fn grep(cfg: Config) {
                     multiple_files,
                     invert,
                     i,
-                    &cleaned.to_owned(),
+                    cleaned,
                     "stdin",
                     &re,
-                ) {
-                    if max > 0 {
-                        matches += 1;
-                        if matches >= max {
-                            break;
-                        }
+                ) && max > 0
+                {
+                    matches += 1;
+                    if matches >= max {
+                        break;
                     }
                 }
 
@@ -257,15 +257,15 @@ fn grep(cfg: Config) {
             return;
         }
 
-        if filenames.len() == 0 {
+        if filenames.is_empty() {
             error("No files specified");
         }
 
+        let mut line = String::new();
         for filename in &filenames {
             let mut matches: u32 = 0;
-            let reader = &mut read_file(&filename);
+            let reader = &mut read_file(filename);
 
-            let mut line = String::new();
             let mut i = 0;
             while let Ok(bytes) = reader.read_line(&mut line) {
                 if bytes == 0 {
@@ -278,15 +278,14 @@ fn grep(cfg: Config) {
                     multiple_files,
                     invert,
                     i,
-                    &cleaned.to_owned(),
+                    cleaned,
                     filename,
                     &re,
-                ) {
-                    if max > 0 {
-                        matches += 1;
-                        if matches >= max {
-                            break;
-                        }
+                ) && max > 0
+                {
+                    matches += 1;
+                    if matches >= max {
+                        break;
                     }
                 }
 
@@ -308,16 +307,14 @@ fn print_match(
 ) {
     let res = if multiple_files {
         if show_lines {
-            write!(writer, "{}:{}:{}\n", filename, index + 1, line)
+            writeln!(writer, "{}:{}:{}", filename, index + 1, line)
         } else {
-            write!(writer, "{}:{}\n", filename, line)
+            writeln!(writer, "{}:{}", filename, line)
         }
+    } else if show_lines {
+        writeln!(writer, "{}:{}", index + 1, line)
     } else {
-        if show_lines {
-            write!(writer, "{}:{}\n", index + 1, line)
-        } else {
-            write!(writer, "{}\n", line)
-        }
+        writeln!(writer, "{}", line)
     };
     if let Err(e) = res {
         error(&format!("Error writing to stdout: {}", e));
@@ -341,7 +338,7 @@ fn error(message: &str) {
 }
 
 fn clean_string(s: &str) -> &str {
-    &s.trim_end_matches(|c| c == '\n' || c == '\r')
+    s.trim_end_matches(|c| c == '\n' || c == '\r')
 }
 
 fn check_string(
@@ -352,17 +349,17 @@ fn check_string(
     case_insensitive: bool,
     match_on: MatchOn,
     i: usize,
-    line: &String,
+    line: &str,
     source: &str,
-    pattern: &String,
+    pattern: &str,
 ) -> bool {
     let line = if case_insensitive {
-        line.to_lowercase()
+        Cow::Owned(line.to_lowercase())
     } else {
-        line.to_owned()
+        line.into()
     };
-    if (match_on == MatchOn::Anywhere && line.includes(pattern) ^ invert)
-        || (match_on == MatchOn::Line && (line == pattern.to_owned()) ^ invert)
+    if (match_on == MatchOn::Anywhere && (&*line).includes(pattern) ^ invert)
+        || (match_on == MatchOn::Line && (line == *pattern) ^ invert)
         || (match_on == MatchOn::Word
             && line
                 .split_whitespace()
@@ -380,12 +377,12 @@ fn check_regex(
     multiple_files: bool,
     invert: bool,
     i: usize,
-    line: &String,
+    line: &str,
     source: &str,
     pattern: &Regex,
 ) -> bool {
-    if pattern.is_match(&line) ^ invert {
-        print_match(writer, i, &line, show_lines, source, multiple_files);
+    if pattern.is_match(line) ^ invert {
+        print_match(writer, i, line, show_lines, source, multiple_files);
         return true;
     }
     false
